/*
QProg.cpp
Written by Walter O. Krawec

Copyright (c) 2013 Walter O. Krawec

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "QProg.h"

#ifdef WIN32
#include <conio.h>
#endif

namespace QProg
{
void Object::saveToFile(std::ofstream& f)
{
	int FILE_VERSION=3;
	f.write((char*)&FILE_VERSION, sizeof(int));
}
void State::saveToFile(std::ofstream& f)
{
	int FILE_VERSION=2;
	f.write((char*)&FILE_VERSION, sizeof(int));
	int stateSize = state.size();
	f.write((char*)&stateSize, sizeof(int));

	if(!state.empty())
	{
		std::list <Pair>::iterator Iter;
		for(Iter = state.begin(); Iter != state.end(); ++Iter)
			Iter->object.saveToFile(f);
	}
}
void QProg::preProcess(std::string& text)
{
	if(text.size() == 0) return;
	std::stringstream str;
	bool nonSpace = false;
	int numSpaces = 0;
	unsigned int endIndex = text.size();
	for(unsigned int i=text.size()-1; i>=0; --i)
	{
		if(text[i] != ' ')
		{
			endIndex = i+1;
			break;
		}
	}

	bool skipSpace = false;
	for(unsigned int i=0; i<endIndex; ++i)
	{
		if(text[i] == '(' && nonSpace && numSpaces == 0)
			str << ' ';
		if(text[i] != ' ')
			nonSpace = true;

		if(text[i] != ' ' && text[i] != '\t')
			numSpaces = 0;

		if(nonSpace)
		{
			if((text[i] == ' ' && !skipSpace) || text[i] != ' ')
			str << text[i];
		}
		if(text[i] == ' ')
		{
			++numSpaces;
			nonSpace = false;
		}
		else if(text[i] == '(')
			skipSpace = true;
		else
			skipSpace = false;
	}
	text = std::string(str.str());
	//std::cout << "PRE-PROCESS = '" << text << "'\n";
}

void QProg::evaluate(const std::string& line, State& state, Object& output, bool globalMode, bool copyState)
{
	wok::Text text;
	text.addIgnore('\t');
	text.addDelimiter(' ');
	text.addDelimiter(',');
	text.addCollection(wok::Text::Pair('(', ')'));
	text.addCollection('"');

	text.parse(line);
	if(text.size() == 0)
		return;
	if(text.size() == 1)
	{
		if(text[0].size() > 0 && ((text[0][0] >= '0' && text[0][0] <= '9') || (text[0][0] == '-' && text[0].size() > 1) || (text[0][0] == '.' && text[0].size() > 1)))
		{
			// cast to integer or double:
			bool integerType = true;
			bool numberType = true;
			for(int j=0; j<(int)text[0].size(); ++j)
			{
				if(text[0][j] == '.')
				{
					if(!integerType)
					{
						numberType = false;
						break;
					}
					integerType = false;
				}
			}

			if(!numberType)
				ObjectHelper::newString(text[0], output);
			else if(integerType)
				ObjectHelper::newInteger(atoi(text[0].c_str()), output);
			else
				ObjectHelper::newDouble(atof(text[0].c_str()), output);
		}
		else
			ObjectHelper::newString(text[0], output);
		return;
	}
	
	State stateCopy;

	state.copy(stateCopy);

	State* ourState = &stateCopy;

	if(!copyState)
		ourState = &state;

	if(globalMode)
		ourState = &globalState;

	if(text[0] == "global")
	{
		evaluate(text[1], stateCopy, output, true);
	}
	else if(text[0] == "begin")
	{
		for(unsigned int i=1; i<text.size(); ++i)
		{
			evaluate(text[i], stateCopy, output, globalMode, false);
		}
	}
	else if(text[0] == "while")
	{
		while(1)
		{
#ifdef WIN32
			if(_kbhit()) throw std::string("Error: User break.");
#endif

			Object condition;
			evaluate(text[1], stateCopy, condition, globalMode, false);
			if(condition.getType() != "integer")
			{
				std::cout << "Syntax error; condition of while loop not integer\n";
				return;
			}
			if(*ObjectHelper::getInteger(condition) == 0)
				break;

			for(unsigned int i=2; i<text.size(); ++i)
			{
				evaluate(text[i], stateCopy, condition, globalMode, false);
			}
		}
	}
	else if(text[0] == "nop" || text[0] == "NOP")
	{
	}
	else if(text[0] == "rootOfUnity")
	{
		Object r;
		evaluate(text[1], stateCopy, r, globalMode);
		Complex result;
		double pi = 3.1415926535897932384626433832795;
		result = algebra::ComplexHelper::construct(2.0*pi/(double)(*ObjectHelper::getInteger(r)));
		ObjectHelper::newComplex(result, output);
	}
	else if(text[0] == "rand")
	{
		// rand (max)
		Object maxRange;
		evaluate(text[1], stateCopy, maxRange, globalMode);
		int maxRange_int = *ObjectHelper::getInteger(maxRange);
		if(maxRange_int <= 0)
		{
			std::cout << "Syntax Error '" << line << "'; maxRange <= 0\n";
			return;
		}

		int result = rand()%maxRange_int;
		ObjectHelper::newInteger(result, output);
	}
	else if(text[0] == "randf")
	{
		// randf (min) (max)
		Object o;
		evaluate(text[1], stateCopy, o, globalMode);
		double minRange = *ObjectHelper::getDouble(o);
		evaluate(text[2], stateCopy, o, globalMode);
		double maxRange = *ObjectHelper::getDouble(o);

		int result = rand()%10000;
		double r = (double)result;
		r = r / 10000.0;
		r = (maxRange - minRange)*r + minRange;
		ObjectHelper::newDouble(r, output);
	}
	else if(text[0] == "cast")
	{
		// cast new_type (object)
		Object type, o;
		evaluate(text[1], stateCopy, type, globalMode);
		evaluate(text[2], stateCopy, o, globalMode);
		if(*ObjectHelper::getString(type) == "complex")
		{
			if(o.getType() == "double")
				ObjectHelper::newComplex(*ObjectHelper::getDouble(o), 0, output);
			else
				std::cout << "Syntax error '" << line << "'; cast argument type not recognized.\n";
		}
	}
	else if(text[0] == "neg")
	{
		Object o;
		evaluate(text[1], stateCopy, o, globalMode);
		if(o.getType() == "integer")
			ObjectHelper::newInteger(-1 * (*ObjectHelper::getInteger(o)), output);
		else if(o.getType() == "double")
			ObjectHelper::newDouble(-1.0 * (*ObjectHelper::getDouble(o)), output);
		else if(o.getType() == "complex")
		{
			Complex c = *ObjectHelper::getComplex(o);
			c.real *= -1.0;
			c.imag *= -1.0;
			ObjectHelper::newComplex(c, output);
		}
	}
	else if(text[0] == "is_unitary")
	{
		Object mat;
		evaluate(text[1], stateCopy, mat, globalMode);
		if(mat.getType() != "matrix")
			std::cout << "Syntax error '" << line << "'; type not matrix (isUnitary)\n";

		algebra::mat *m = ObjectHelper::getMatrix(mat);
		algebra::mat m2, m3;
		m->transpose(m2);
		m->multiply(m2, m3);
		int result = 1;
		double t = 0.0001;
		//m3.print(std::cout, true);
		for(int i=0; i<m3.getRows(); ++i)
		{
			for(int j=0; j<m3.getCols(); ++j)
			{
				if(i == j && fabs(m3(i,j).real - 1.0) > t || fabs(m3(i,j).imag - 0.0) > t)
					result = 0;
				else if(i != j && fabs(m3(i,j).real - 0.0) > t || fabs(m3(i,j).imag - 0.0) > t)
					result = 0;
			}
		}

		ObjectHelper::newInteger(result, output);
	}
	else if(text[0] == "transpose")
	{
		Object mat;
		evaluate(text[1], stateCopy, mat, globalMode);
		if(mat.getType() != "matrix")
			std::cout << "Syntax error '" << line << "'; type not matrix (transpose)\n";
		algebra::mat *m = ObjectHelper::getMatrix(mat);
		algebra::mat *m2 = new algebra::mat;
		m->transpose(*m2);
		ObjectHelper::newMatrix(m2, output);
	}
	else if(text[0] == "gate")
	{
		// gate type size [args]
		Object name, dim;
		evaluate(text[1], stateCopy, name, globalMode);
		evaluate(text[2], stateCopy, dim, globalMode);

		int dimSize = *ObjectHelper::getInteger(dim);
		if(dimSize <= 0)
		{
			std::cout << "Syntax Error '" << line << "'; dimSize <= 0\n";
			return;
		}
		algebra::mat* m = new algebra::mat;
		int k = 1;
		for(int i=0; i<dimSize; ++i)
			k = k << 1;
		m->create(k);
		m->zero();
		ObjectHelper::newMatrix(m, output);

		if(*ObjectHelper::getString(name) == "cnot")
		{
			Object controlObj, actionObj;
			evaluate(text[3], stateCopy, controlObj, globalMode);
			evaluate(text[4], stateCopy, actionObj, globalMode);
			int control=1, action=1;
			for(int i=0; i<*ObjectHelper::getInteger(controlObj); ++i)
				control = control << 1;
			for(int i=0; i<*ObjectHelper::getInteger(actionObj); ++i)
				action = action << 1;

			//std::cout << "control = " << control << "\naction = " << action << "\n";

			if(action == control)
			{
				std::cout << "Syntax Error '" << line << "'; action = control\n";
				return;
			}

			int i = 1;
			for(int c=0; c<k; ++c)
			{
				int thisMap = c;
				if( (c&control) / control == 1 )
				{
					// control bit is 1 so should map c -> +- action
					if( (c&action) / action == 1 )
						thisMap -= action;
					else
						thisMap += action;
				}
				int j = 1;
				for(int r=0; r<k; ++r)
				{
					if(r == thisMap)
						(*m)(r,c) = algebra::ComplexHelper::one();

					j = j << 1;
				}

				i = i << 1;
			}
		}
	}
	else if(text[0] == "sin")
	{
		Object t;
		evaluate(text[1], stateCopy, t, globalMode);
		double result = sin(*ObjectHelper::getDouble(t));

		ObjectHelper::newDouble(result, output);
	}
	else if(text[0] == "cos")
	{
		Object t;
		evaluate(text[1], stateCopy, t, globalMode);
		double result = cos(*ObjectHelper::getDouble(t));

		ObjectHelper::newDouble(result, output);
	}
	else if(text[0] == "inv")
	{
		Object e;
		evaluate(text[1], stateCopy, e, globalMode);
		double d=1;
		if(e.getType() == "integer")
			d = (double) (*ObjectHelper::getInteger(e));
		else
			d = *ObjectHelper::getDouble(e);

		ObjectHelper::newDouble(1.0 / d, output);
	}
	else if(text[0] == "sqrt")
	{
		Object e;
		evaluate(text[1], stateCopy, e, globalMode);
		double result = 1;
		if(e.getType() == "integer")
			result = (double) (*ObjectHelper::getInteger(e));
		else
			result = *ObjectHelper::getDouble(e);
		ObjectHelper::newDouble(sqrt(result), output);
	}
	else if(text[0] == "pow2")
	{
		Object e;
		evaluate(text[1], stateCopy, e, globalMode);
		int e_int = *ObjectHelper::getInteger(e);

		int k = 1;
		for(int i=0; i<e_int; ++i)
			k = k << 1;

		ObjectHelper::newInteger(k, output);
	}
	else if(text[0] == "pow")
	{
		Object b, e;
		evaluate(text[1], stateCopy, b, globalMode);
		evaluate(text[2], stateCopy, e, globalMode);
		if(b.getType() == "matrix")
		{
		}
		else if(b.getType() == "complex")
		{
			Complex result;
			result = algebra::ComplexHelper::power(*ObjectHelper::getComplex(b), *ObjectHelper::getInteger(e));
			ObjectHelper::newComplex(result, output);
		}
		else if(b.getType() == "integer" && e.getType() == "integer")
		{
			int result = 1;
			for(int i=0; i<*ObjectHelper::getInteger(e); ++i)
				result *= (*ObjectHelper::getInteger(b));

			ObjectHelper::newInteger(result, output);
		}
		else
		{
			ObjectHelper::newDouble(pow(*ObjectHelper::getDouble(b), *ObjectHelper::getDouble(e)), output);
		}
	}
	else if(text[0] == "powm")
	{
		Object b, e, m;
		evaluate(text[1], stateCopy, b, globalMode);
		evaluate(text[2], stateCopy, e, globalMode);
		evaluate(text[3], stateCopy, m, globalMode);
		int result = 1;
		for(int i=0; i<*ObjectHelper::getInteger(e); ++i)
		{
			result *= (*ObjectHelper::getInteger(b));
			result = result % (*ObjectHelper::getInteger(m));
		}

		ObjectHelper::newInteger(result, output);
	}
	else if(text[0] == "exp")
	{
		Object e;
		evaluate(text[1], stateCopy, e, globalMode);
		if(e.getType() == "complex")
		{
			double re = exp(ObjectHelper::getComplex(e)->real);
			Complex c;
			c = algebra::ComplexHelper::construct(ObjectHelper::getComplex(e)->imag);

			c.real *= re;
			c.imag *= re;

			ObjectHelper::newComplex(c, output);
		}
	}
	else if(text[0] == "comp.less" || text[0] == "comp.lt" || text[0] == "<")
	{
		Object val1, val2;
		evaluate(text[1], stateCopy, val1, globalMode);
		evaluate(text[2], stateCopy, val2, globalMode);

		if(val1.getType() != val2.getType())
		{
			std::cout << "Cannot compare different types; '" << line << "'\n";
			return;
		}

		if(val1.getType() == "integer")
		{
			if(*ObjectHelper::getInteger(val1) < *ObjectHelper::getInteger(val2))
				ObjectHelper::newInteger(1, output);
			else
				ObjectHelper::newInteger(0, output);
		}
		else if(val1.getType() == "double")
		{
			if(*ObjectHelper::getDouble(val1) < *ObjectHelper::getDouble(val2))
				ObjectHelper::newInteger(1, output);
			else
				ObjectHelper::newInteger(0, output);
		}
		else
		{
			std::cout << "Cannot compare these types; '" << line << "'\n";
			return;
		}
	}
	else if(text[0] == "comp.lte")
	{
		Object val1, val2;
		evaluate(text[1], stateCopy, val1, globalMode);
		evaluate(text[2], stateCopy, val2, globalMode);

		if(val1.getType() != val2.getType())
		{
			std::cout << "Cannot compare different types; '" << line << "'\n";
			return;
		}

		if(val1.getType() == "integer")
		{
			if(*ObjectHelper::getInteger(val1) <= *ObjectHelper::getInteger(val2))
				ObjectHelper::newInteger(1, output);
			else
				ObjectHelper::newInteger(0, output);
		}
		else if(val1.getType() == "double")
		{
			if(*ObjectHelper::getDouble(val1) <= *ObjectHelper::getDouble(val2))
				ObjectHelper::newInteger(1, output);
			else
				ObjectHelper::newInteger(0, output);
		}
		else
		{
			std::cout << "Cannot compare these types; '" << line << "'\n";
			return;
		}
	}
	else if(text[0] == "comp.gt" || text[0] == ">")
	{
		Object val1, val2;
		evaluate(text[1], stateCopy, val1, globalMode);
		evaluate(text[2], stateCopy, val2, globalMode);

		if(val1.getType() != val2.getType())
		{
			std::cout << "Cannot compare different types; '" << line << "'\n";
			return;
		}

		if(val1.getType() == "integer")
		{
			if(*ObjectHelper::getInteger(val1) > *ObjectHelper::getInteger(val2))
				ObjectHelper::newInteger(1, output);
			else
				ObjectHelper::newInteger(0, output);
		}
		else if(val1.getType() == "double")
		{
			if(*ObjectHelper::getDouble(val1) > *ObjectHelper::getDouble(val2))
				ObjectHelper::newInteger(1, output);
			else
				ObjectHelper::newInteger(0, output);
		}
		else
		{
			std::cout << "Cannot compare these types; '" << line << "'\n";
			return;
		}
	}
	else if(text[0] == "comp.gte")
	{
		Object val1, val2;
		evaluate(text[1], stateCopy, val1, globalMode);
		evaluate(text[2], stateCopy, val2, globalMode);

		if(val1.getType() != val2.getType())
		{
			std::cout << "Cannot compare different types; '" << line << "'\n";
			return;
		}

		if(val1.getType() == "integer")
		{
			if(*ObjectHelper::getInteger(val1) >= *ObjectHelper::getInteger(val2))
				ObjectHelper::newInteger(1, output);
			else
				ObjectHelper::newInteger(0, output);
		}
		else if(val1.getType() == "double")
		{
			if(*ObjectHelper::getDouble(val1) >= *ObjectHelper::getDouble(val2))
				ObjectHelper::newInteger(1, output);
			else
				ObjectHelper::newInteger(0, output);
		}
		else
		{
			std::cout << "Cannot compare these types; '" << line << "'\n";
			return;
		}
	}
	else if(text[0] == "comp.ne" || text[0] == "!=")
	{
		Object val1, val2;
		evaluate(text[1], stateCopy, val1, globalMode);
		evaluate(text[2], stateCopy, val2, globalMode);

		if(val1.getType() != val2.getType())
		{
			std::cout << "Cannot compare different types; '" << line << "'\n";
			return;
		}

		if(val1.getType() == "integer")
		{
			if(*ObjectHelper::getInteger(val1) != *ObjectHelper::getInteger(val2))
				ObjectHelper::newInteger(1, output);
			else
				ObjectHelper::newInteger(0, output);
		}
		else if(val1.getType() == "double")
		{
			if(*ObjectHelper::getDouble(val1) != *ObjectHelper::getDouble(val2))
				ObjectHelper::newInteger(1, output);
			else
				ObjectHelper::newInteger(0, output);
		}
		else
		{
			std::cout << "Cannot compare these types; '" << line << "'\n";
			return;
		}
	}
	else if(text[0] == "comp.eq" || text[0] == "==")
	{
		Object val1, val2;
		evaluate(text[1], stateCopy, val1, globalMode);
		evaluate(text[2], stateCopy, val2, globalMode);

		if(val1.getType() != val2.getType())
		{
			std::cout << "Cannot compare different types; '" << line << "'\n";
			return;
		}

		if(val1.getType() == "integer")
		{
			if(*ObjectHelper::getInteger(val1) == *ObjectHelper::getInteger(val2))
				ObjectHelper::newInteger(1, output);
			else
				ObjectHelper::newInteger(0, output);
		}
		else if(val1.getType() == "double")
		{
			if(*ObjectHelper::getDouble(val1) == *ObjectHelper::getDouble(val2))
				ObjectHelper::newInteger(1, output);
			else
				ObjectHelper::newInteger(0, output);
		}
		else
		{
			std::cout << "Cannot compare these types; '" << line << "'\n";
			return;
		}
	}
	else if(text[0] == "if")
	{
		Object test;
		evaluate(text[1], stateCopy, test, globalMode);
		try
		{
			if(*ObjectHelper::getInteger(test) != 0)
				evaluate(text[2], stateCopy, output, globalMode);
			else
				evaluate(text[3], stateCopy, output, globalMode);
		}
		catch(std::string& e)
		{
			std::cout << "Syntax error '" << line << "': " << e << "\n";
		}
	}
	else if(text[0] == "user.input")
	{
		// user.input TYPE
		char buffer[101];
		Object type_obj;
		evaluate(text[1], stateCopy, type_obj, globalMode);
		if(*ObjectHelper::getString(type_obj) == "integer")
		{
			std::cout << "Please enter an integer: ";
			std::cin.getline(buffer, 100);
			ObjectHelper::newInteger(atoi(buffer), output);
		}
		else
		{
			std::cout << "user.input: ";
			std::cin.getline(buffer, 100);
		}
		std::cout << "\n";
	}
	else if(text[0] == "file")
	{
		Object filename;
		std::ifstream f;
		f.open(text[1].c_str());
		if(!f.is_open())
		{
			std::cout << "Cannot open file '" << text[1] << "'\n";
			return;
		}

		int maxBuffer = 2048;
		char* buffer = new char[maxBuffer+2];
		std::stringstream str;
		while(!f.eof())
		{
			char c = f.get();
			if(c == 10 || c == 13)
				str << " ";
			else
				str << c;
		}

		std::string newLine(str.str());
		preProcess(newLine);

		evaluate(newLine, stateCopy, output, globalMode, false);

		delete[] buffer;
	}
	else if(text[0] == "delete")
	{
		Object name;
		evaluate(text[1], stateCopy, name, globalMode);
		state.removeState(*ObjectHelper::getString(name));
	}
	else if(text[0] == "new")
	{
		Object type, name;
		evaluate(text[1], stateCopy, type, globalMode);
		evaluate(text[2], stateCopy, name, globalMode);

		if(*ObjectHelper::getString(name) == "temp")
		{
			std::cout << "Illegal name 'temp'; on line '" << line << "'\n";
			return;
		}

		if(*ObjectHelper::getString(type) == "space")
		{
			Object dim;
			evaluate(text[3], stateCopy, dim, globalMode);
			int d = *ObjectHelper::getInteger(dim);

			if(d <= 0)
			{
				std::cout << "Syntax error - dim <= 0\n";
				return;
			}

			std::string nameStr(*ObjectHelper::getString(name));
			Complex one;
			one.real = 1;
			one.imag = 0;
			for(int i=0; i<d; ++i)
			{
				std::stringstream str, str2;
				str << nameStr << ".ket." << i;
				str2 << nameStr << ".bra." << i;
				algebra::mat *ket = new algebra::mat;
				ket->create(d, 1);
				ket->zero();
				ket->setDirect(i, 0, one);

				Object o(str.str(), "matrix", ket);
				ourState->addState(o);

				algebra::mat *bra = new algebra::mat;
				bra->create(1, d);
				bra->zero();
				bra->setDirect(0, i, one);

				Object o2(str2.str(), "matrix", bra);
				ourState->addState(o2);
			}

			std::stringstream str;
			str << nameStr << ".size";
			int *i = new int;
			*i = d;
			Object o3(str.str(), "integer", i);
			ourState->addState(o3);
		}
		else if(*ObjectHelper::getString(type) == "system")
		{
			Object n;
			std::vector <int> *v;
			v = new std::vector <int>;
			evaluate(text[3], stateCopy, n, globalMode);
			for(int i=0; i<*ObjectHelper::getInteger(n); ++i)
			{
				Object space, spaceSize;
				evaluate(text[4+i], stateCopy, space, globalMode);
				std::stringstream t;
				t << (*ObjectHelper::getString(space)) << ".size";
				ourState->get(t.str(), spaceSize);
				v->push_back(*ObjectHelper::getInteger(spaceSize));
			}

			Object o(ObjectHelper::getString(name), ObjectHelper::getString(type), v);
			ourState->addState(o);

			std::stringstream str;
			str << *ObjectHelper::getString(name) << ".size";
			int *i = new int;
			*i = (int)v->size();
			Object o2(str.str(), "integer", i);
			ourState->addState(o2);
		}
		else if(*ObjectHelper::getString(type) == "matrix")
		{
			Object rows, cols;
			evaluate(text[3], stateCopy, rows, globalMode);
			evaluate(text[4], stateCopy, cols, globalMode);

			int r = *ObjectHelper::getInteger(rows);
			int c = *ObjectHelper::getInteger(cols);
			if(r <= 0 || c <= 0)
			{
				std::cout << "Syntax error create matrix\n";
				return;
			}
			algebra::mat * m = new algebra::mat;
			m->create(r, c);
			m->zero();

			Object o(ObjectHelper::getString(name), ObjectHelper::getString(type), m);
			ourState->addState(o);
		}
		else
		{
			std::string type_str(*ObjectHelper::getString(type));
			int amount = 1;
			if(*ObjectHelper::getString(type) == "array")
			{
				// (new array NAME TYPE DIM)
				Object type_obj, dim_obj;
				evaluate(text[3], stateCopy, type_obj, globalMode);
				evaluate(text[4], stateCopy, dim_obj, globalMode);

				amount = *ObjectHelper::getInteger(dim_obj);
				type_str = *ObjectHelper::getString(type_obj);

				int *newInt = new int;
				*newInt = amount;
				std::stringstream str;
				str << *ObjectHelper::getString(name) << ".size";
				Object o(str.str(), "integer", newInt);
				ourState->addState(o);
			}

			for(int i=0; i<amount; ++i)
			{
				std::stringstream name_str;
				name_str << *ObjectHelper::getString(name);
				if(amount > 1)
					name_str << "." << i;
				if(type_str == "integer")
				{
					int *newInt = new int;
					*newInt = 0;

					Object o(name_str.str(), "integer", newInt);
					ourState->addState(o);
				}
				else if(type_str == "double")
				{
					double *newDouble = new double;
					*newDouble = 0;
					/**newDouble = rand()%10000;
					*newDouble /= 10000.0;
					*newDouble *= 3.14/2.0;*/

					Object o(name_str.str(), "double", newDouble);
					ourState->addState(o);
				}
				else if(type_str == "complex")
				{
					Complex *newC = new Complex;
					*newC = algebra::ComplexHelper::zero();

					Object o(name_str.str(), "complex", newC);
					ourState->addState(o);
				}
				else if(type_str == "matrix")
				{
					algebra::mat *m = new algebra::mat();
					m->create(1);
					Object o(name_str.str(), "matrix", m);
					ourState->addState(o);
				}
				else
				{
					std::cout << "Syntax error '" << line << "'; illegal type (or cannot be used as array)\n";
					throw std::string("Error.");
				}
			}
		}
	}
	else if(text[0] == "index")
	{
		Object name, index;
		evaluate(text[1], stateCopy, name, globalMode);
		std::stringstream arrayName;
		arrayName << *ObjectHelper::getString(name);
		for(int i=2; i<(int)text.size(); ++i)
		{
			evaluate(text[i], stateCopy, index, globalMode);

			if(name.getType() != "string")
			{
				std::cout << "Syntax error '" << line << "'\n";
				return;
			}

			
			if(index.getType() == "integer")
				arrayName << "." << *ObjectHelper::getInteger(index);
			else if(index.getType() == "string")
			{
				if(ObjectHelper::getString(index)->size() > 0 && (*ObjectHelper::getString(index))[0] != ' ')
					arrayName << "." << *ObjectHelper::getString(index);
			}
			else
			{
				std::cout << "Syntax error '" << line << "'; index not integer or string.\n";
				return;
			}
		}
		ObjectHelper::newString(arrayName.str(), output);
	}
	else if(text[0] == "get_index")
	{
		Object name, index;
		evaluate(text[1], stateCopy, name, globalMode);
		evaluate(text[2], stateCopy, index, globalMode);

		if(name.getType() != "string" || index.getType() != "integer")
		{
			std::cout << "Syntax error '" << line << "'\n";
			return;
		}

		std::stringstream arrayName;
		arrayName << *ObjectHelper::getString(name) << "." << *ObjectHelper::getInteger(index);
		ourState->get(arrayName.str(), output);
	}
	else if(text[0] == "integer")
	{
		Object valueStr;
		evaluate(text[1], stateCopy, valueStr, globalMode);
		if(valueStr.getType() == "integer")
			ObjectHelper::newInteger(*ObjectHelper::getInteger(valueStr), output);
		else
			ObjectHelper::newInteger(atoi(ObjectHelper::getString(valueStr)->c_str()), output);
	}
	else if(text[0] == "double")
	{
		Object valueStr;
		evaluate(text[1], stateCopy, valueStr, globalMode);
		if(valueStr.getType() == "double")
			ObjectHelper::newDouble(*ObjectHelper::getDouble(valueStr), output);
		else
			ObjectHelper::newDouble(atof(ObjectHelper::getString(valueStr)->c_str()), output);
	}
	else if(text[0] == "real")
	{
		Object c;
		evaluate(text[1], stateCopy, c, globalMode);
		ObjectHelper::newDouble(ObjectHelper::getComplex(c)->real, output);
	}
	else if(text[0] == "complex")
	{
		Object realStr, imagStr;
		evaluate(text[1], stateCopy, realStr, globalMode);
		evaluate(text[2], stateCopy, imagStr, globalMode);
		double re = 0, im = 0;
		if(realStr.getType() == "string")
			re = atof(ObjectHelper::getString(realStr)->c_str());
		else if(realStr.getType() == "double")
			re = *ObjectHelper::getDouble(realStr);
		else if(realStr.getType() == "integer")
			re = *ObjectHelper::getInteger(realStr);
		else
		{
			std::cout << "Syntax error '" << line << "'\n";
			return;
		}

		if(imagStr.getType() == "string")
			im = atof(ObjectHelper::getString(imagStr)->c_str());
		else if(imagStr.getType() == "double")
			im = *ObjectHelper::getDouble(imagStr);
		else if(imagStr.getType() == "integer")
			im = *ObjectHelper::getInteger(imagStr);
		else
		{
			std::cout << "Syntax error '" << line << "'\n";
			return;
		}
		ObjectHelper::newComplex(re, im, output);
		//ObjectHelper::newComplex(atof(ObjectHelper::getString(realStr)->c_str()), atof(ObjectHelper::getString(imagStr)->c_str()), output);
	}
	else if(text[0] == "cnorm")
	{
		Object c;
		evaluate(text[1], stateCopy, c, globalMode);
		Complex num1, num2;
		if(c.getType() == "matrix")
		{
			algebra::mat *m = ObjectHelper::getMatrix(c);
			if(m->getSize() != 1)
			{
				std::cout << "Matrix not size 1 in cnorm; '" << line << "'\n";
				return;
			}

			num1 = m->getDirect(0,0);
		}
		else
			num1 = *ObjectHelper::getComplex(c);
		num2.real = num1.real;
		num2.imag = -num1.imag;

		num1 = algebra::ComplexHelper::mult(num1, num2);

		ObjectHelper::newComplex(num1, output);
	}
	else if(text[0] == "vector")
	{
		std::cout << "create vector of size " << text.size()-1 << "\n";
	}
	else if(text[0] == "rowSize")
	{
		Object o;
		evaluate(text[1], stateCopy, o, globalMode);
		ObjectHelper::newInteger(ObjectHelper::getMatrix(o)->getRows(), output);
	}
	else if(text[0] == "colSize")
	{
		Object o;
		evaluate(text[1], stateCopy, o, globalMode);
		ObjectHelper::newInteger(ObjectHelper::getMatrix(o)->getCols(), output);
	}
	else if(text[0] == "random_int")
	{
		Object small, large;
		evaluate(text[1], stateCopy, small, globalMode);
		evaluate(text[2], stateCopy, large, globalMode);

		int small_int = *ObjectHelper::getInteger(small);
		int large_int = *ObjectHelper::getInteger(large);
		int delta = large_int - small_int;
		if(delta < 0)
		{
			std::cout << "Syntax error '" << line << "'; large < small\n";
			throw std::string("Syntax error.");
		}

		int result = rand()%100*(delta+1);
		result += (small_int*100);
		result /= 100;
		//int result = rand()%10*(delta+1)/10 + small_int;
		ObjectHelper::newInteger(result, output);
	}
	else if(text[0] == "matrix")
	{
		Object rowStr, colStr;
		evaluate(text[1], stateCopy, rowStr, globalMode);
		evaluate(text[2], stateCopy, colStr, globalMode);

		int rows = *ObjectHelper::getInteger(rowStr);
		int cols = *ObjectHelper::getInteger(colStr);

		algebra::mat *m = ObjectHelper::newMatrix(rows, cols, output);

		int counter = 3;
		for(int r=0; r<rows; ++r)
		{
			for(int c=0; c<cols; ++c)
			{
				Object element;
				evaluate(text[counter], stateCopy, element, globalMode);
				if(element.getType() == "string" && r == 0 && c == 0 && *ObjectHelper::getString(element) == "ident")
				{
					int dim = rows;
					if(cols < rows) dim = cols;
					m->clear(algebra::ComplexHelper::zero());
					for(; r<dim; ++r)
						m->setDirect(r,r, algebra::ComplexHelper::one());
					r = rows;
					c = cols;
					break;
				}
				else if(element.getType() == "string" && r == 0 && c == 0 && *ObjectHelper::getString(element) == "zero")
				{
					int dim = rows;
					if(cols < rows) dim = cols;
					m->clear(algebra::ComplexHelper::zero());
					r = rows;
					c = cols;
					break;
				}
				else if(element.getType() == "string" && r == 0 && c == 0 && *ObjectHelper::getString(element) == "u_random")
				{
					// matrix DIM DIM u_random phi psi chi
					Object phi_obj, psi_obj, chi_obj;
					evaluate(text[4], stateCopy, phi_obj, globalMode);
					evaluate(text[5], stateCopy, psi_obj, globalMode);
					evaluate(text[6], stateCopy, chi_obj, globalMode);
					int dim = rows;
					algebra::mat E, temp, temp2;
					//std::vector <algebra::mat> E;
					//E.resize(dim);
					//E = new algebra::mat[dim];
					temp.create(dim);
					int index = 0;
					Complex z;
					m->ident();
					for(int j=1; j<=dim-1; ++j)
					{
						E.create(dim);
						E.ident();
						for(int i=j; i>=1; --i)
						{
							int I = i;
							int J = j+1;
							Object o;
							double psi, phi, chi;
							{
								std::stringstream str;
								str << *ObjectHelper::getString(psi_obj) << "." << index;
								ourState->get(str.str(), o);
								std::cout << str.str() << "\n";
								psi = *ObjectHelper::getDouble(o);
								o.clear();
							}
							{
								std::stringstream str;
								str << *ObjectHelper::getString(phi_obj) << "." << index;
								ourState->get(str.str(), o);
								std::cout << str.str() << "\n";
								phi = *ObjectHelper::getDouble(o);
								o.clear();
							}
							if(I == 1)
							{
								std::stringstream str;
								str << *ObjectHelper::getString(chi_obj) << "." << i-1;
								ourState->get(str.str(), o);
								std::cout << str.str() << "\n";
								chi = *ObjectHelper::getDouble(o);
								o.clear();
							}
							else
								chi = 0;
							++index;
							temp.ident();
							z = algebra::ComplexHelper::construct(psi);
							z.real *= cos(phi);
							z.imag *= cos(phi);
							temp(I-1, I-1) = z;

							z = algebra::ComplexHelper::construct(chi);
							z.real *= sin(phi);
							z.imag *= sin(phi);
							temp(I-1, J-1) = z;

							z = algebra::ComplexHelper::construct(-chi);
							z.real *= -sin(phi);
							z.imag *= -sin(phi);
							temp(J-1, I-1) = z;

							z = algebra::ComplexHelper::construct(-psi);
							z.real *= cos(phi);
							z.imag *= cos(phi);
							temp(J-1, J-1) = z;

							E.multiply(temp, temp2);
							E = temp2;
						}

						m->multiply(E, temp2);
						*m = temp2;
					}
					r = rows;
					c = cols;
					break;
				}
				else if(element.getType() == "string" && r == 0 && c == 0 && (*ObjectHelper::getString(element) == "UFT" || *ObjectHelper::getString(element) == "QFT"))
				{
					for(; r<rows; ++r)
					{
						for(c=0; c<cols; ++c)
						{
							Complex rt;
							double pi = 3.1415926535897932384626433832795;
							rt = algebra::ComplexHelper::construct(2.0*pi/(double)(m->getRows()));
							(*m)(r,c) = algebra::ComplexHelper::power(rt, r*c);
						}
					}

					Complex scale;
					scale.real = 1.0 / sqrt((double)rows);
					scale.imag = 0;
					m->multiplyScalar(scale);
					r = rows;
					c = cols;
					break;
				}
				try{
					m->setDirect(r, c, *ObjectHelper::getComplex(element));
				}
				catch(std::string& e){std::cout << e << "\n"; return;}
				++counter;
			}
		}
	}
	else if(text[0] == "get")
	{
		Object name;
		evaluate(text[1], stateCopy, name, globalMode);
		ourState->get(*ObjectHelper::getString(name), output);
	}
	else if(text[0] == "size")
	{
		Object name;
		evaluate(text[1], stateCopy, name, globalMode);
		std::stringstream str;
		str << *ObjectHelper::getString(name) << ".size";
		ourState->get(str.str(), output);
	}
	else if(text[0] == "set")
	{
		Object name, value;
		evaluate(text[1], stateCopy, name, globalMode);
		evaluate(text[2], stateCopy, value, globalMode);

		ourState->set(*ObjectHelper::getString(name), value);
	}
	else if(text[0] == "add" || text[0] == "+")
	{
		Object int1, int2;
		evaluate(text[1], stateCopy, int1, globalMode);
		evaluate(text[2], stateCopy, int2, globalMode);

		if(int1.getType() != int2.getType())
		{
			std::cout << "Syntax error '" << line << "'; types don't match\nint1: ";
			int1.print(std::cout, true);
			std::cout << "int2: ";
			int2.print(std::cout, true);
			std::cout << "\n";
		}
		else if(int1.getType() == "integer")
			ObjectHelper::newInteger(*ObjectHelper::getInteger(int1) + *ObjectHelper::getInteger(int2), output);
		else if(int1.getType() == "double")
			ObjectHelper::newDouble(*ObjectHelper::getDouble(int1) + *ObjectHelper::getDouble(int2), output);
		else if(int1.getType() == "complex")
			ObjectHelper::newComplex(algebra::ComplexHelper::add(*ObjectHelper::getComplex(int1), *ObjectHelper::getComplex(int2)), output);
		else if(int1.getType() == "matrix")
		{
			algebra::mat *A, *B;
			A = ObjectHelper::getMatrix(int1);
			B = ObjectHelper::getMatrix(int2);
			if(A->getCols() != B->getCols() && A->getRows() != B->getRows())
			{
				std::cout << "Syntax error '" << line << "'; matrix wrong size for add\n";
				return;
			}
			algebra::mat *C = new algebra::mat;
			Complex one = algebra::ComplexHelper::one();
			A->add(B, C);

			ObjectHelper::newMatrix(C, output);
		}
		else
			std::cout << "Syntax error '" << line << "'; cannot add these types\n";
	}
	else if(text[0] == "subtract" || text[0] == "-")
	{
		Object int1, int2;
		evaluate(text[1], stateCopy, int1, globalMode);
		evaluate(text[2], stateCopy, int2, globalMode);

		if(int1.getType() != int2.getType())
			std::cout << "Syntax error '" << line << "'; types don't match\n";
		else if(int1.getType() == "integer")
			ObjectHelper::newInteger(*ObjectHelper::getInteger(int1) - *ObjectHelper::getInteger(int2), output);
		else if(int1.getType() == "double")
			ObjectHelper::newDouble(*ObjectHelper::getDouble(int1) - *ObjectHelper::getDouble(int2), output);
		else if(int1.getType() == "complex")
			ObjectHelper::newComplex(algebra::ComplexHelper::subtract(*ObjectHelper::getComplex(int1), *ObjectHelper::getComplex(int2)), output);
		else if(int1.getType() == "matrix")
		{
			algebra::mat *A, *B;
			A = ObjectHelper::getMatrix(int1);
			B = ObjectHelper::getMatrix(int2);
			if(A->getCols() != B->getCols() && A->getRows() != B->getRows())
			{
				std::cout << "Syntax error '" << line << "'; matrix wrong size for subtract\n";
				return;
			}
			algebra::mat *C = new algebra::mat;
			Complex one = algebra::ComplexHelper::one();
			A->subtract(B, C);

			ObjectHelper::newMatrix(C, output);
		}
		else
			std::cout << "Syntax error '" << line << "'; cannot subtract these types\n";
	}
	else if(text[0] == "div_d")
	{
		Object int1, int2;
		evaluate(text[1], stateCopy, int1, globalMode);
		evaluate(text[2], stateCopy, int2, globalMode);

		if(int1.getType() != int2.getType())
		{
			if(int1.getType() == "double" && int2.getType() == "integer")
			{
				double result = 1;
				result = (double)(*ObjectHelper::getDouble(int1)) / (double)(*ObjectHelper::getInteger(int2));
				ObjectHelper::newDouble(result, output);
			}
			else
				std::cout << "Syntax error '" << line << "'; types don't match\n";
		}
		else
		{
			double result = 1;
			if(int1.getType() == "integer")
				result = (double)(*ObjectHelper::getInteger(int1)) / (double)(*ObjectHelper::getInteger(int2));

			ObjectHelper::newDouble(result, output);
		}
	}
	else if(text[0] == "apply_op")
	{
		// apply_op SYSTEM INDEX MATRIX MATRIX_STATE
		Object system_obj, index_obj, matrix_obj, state_obj;
		evaluate(text[1], stateCopy, system_obj, globalMode);
		evaluate(text[2], stateCopy, index_obj, globalMode);
		evaluate(text[3], stateCopy, matrix_obj, globalMode);
		evaluate(text[4], stateCopy, state_obj, globalMode);

		std::vector <int> *system;
		system = ObjectHelper::getSystem(system_obj);
		int index = *ObjectHelper::getInteger(index_obj);
		algebra::mat *op, *state, temp, temp2, final_op;
		op = ObjectHelper::getMatrix(matrix_obj);
		state = ObjectHelper::getMatrix(state_obj);

		final_op.create(1);
		final_op.ident();
		for(unsigned int i=0; i<system->size(); ++i)
		{
			if(i == index)
			{
				final_op.tensor(op, &temp);
				final_op = temp;
			}
			else
			{
				temp2.create((*system)[i]);
				temp2.ident();
				final_op.tensor(&temp2, &temp);
				final_op = temp;
			}
		}
		if(final_op.getCols() != state->getRows())
		{
			std::cout << "Syntax error '" << line << "'; matrix wrong size for mult\n";
			//std::cout << "A is " << A->getRows() << "x" << A->getCols() << "\n";
			//std::cout << "B is " << B->getRows() << "x" << B->getCols() << "\n";
			return;
		}
		algebra::mat *C = new algebra::mat;
		final_op.multiply(*state, *C);

		ObjectHelper::newMatrix(C, output);
	}
	else if(text[0] == "mult" || text[0] == "*")
	{
		Object int1, int2;
		evaluate(text[1], stateCopy, int1, globalMode);
		evaluate(text[2], stateCopy, int2, globalMode);

		if(int1.getType() != int2.getType())
		{
			if(int1.getType() == "complex" && int2.getType() == "matrix")
			{
				algebra::mat *C = new algebra::mat;
				*C = *ObjectHelper::getMatrix(int2);
				C->multiplyScalar(*ObjectHelper::getComplex(int1));
				ObjectHelper::newMatrix(C, output);
			}
			else if(int1.getType() == "double" && int2.getType() == "matrix")
			{
				algebra::mat *C = new algebra::mat;
				*C = *ObjectHelper::getMatrix(int2);
				Complex c;
				c.real = *ObjectHelper::getDouble(int1);
				c.imag = 0;
				C->multiplyScalar(c);
				ObjectHelper::newMatrix(C, output);
			}
			else if(int1.getType() == "double" && int2.getType() == "integer")
				ObjectHelper::newDouble(*ObjectHelper::getDouble(int1) * (double)(*ObjectHelper::getInteger(int2)), output);
			else
				std::cout << "Syntax error '" << line << "'; types don't match\n";
		}
		else if(int1.getType() == "double")
			ObjectHelper::newDouble(*ObjectHelper::getDouble(int1) * *ObjectHelper::getDouble(int2), output);
		else if(int1.getType() == "integer")
			ObjectHelper::newInteger(*ObjectHelper::getInteger(int1) * *ObjectHelper::getInteger(int2), output);
		else if(int1.getType() == "complex")
			ObjectHelper::newComplex(algebra::ComplexHelper::mult(*ObjectHelper::getComplex(int1), *ObjectHelper::getComplex(int2)), output);
		else if(int1.getType() == "matrix")
		{
			algebra::mat *A, *B;
			A = ObjectHelper::getMatrix(int1);
			B = ObjectHelper::getMatrix(int2);
			if(A->getCols() != B->getRows())
			{
				std::cout << "Syntax error '" << line << "'; matrix wrong size for mult\n";
				std::cout << "A is " << A->getRows() << "x" << A->getCols() << "\n";
				std::cout << "B is " << B->getRows() << "x" << B->getCols() << "\n";
				return;
			}
			algebra::mat *C = new algebra::mat;
			Complex one = algebra::ComplexHelper::one();
			A->multiply(*B, *C);

			ObjectHelper::newMatrix(C, output);
		}
		else
			std::cout << "Syntax error '" << line << "'; cannot multiply these types\n";
	}
	else if(text[0] == "div" || text[0] == "/")
	{
		Object int1, int2;
		evaluate(text[1], stateCopy, int1, globalMode);
		evaluate(text[2], stateCopy, int2, globalMode);

		if(int1.getType() != int2.getType())
		{
			std::cout << "Syntax error '" << line << "'; types don't match\n";
		}
		else if(int1.getType() == "double")
			ObjectHelper::newDouble(*ObjectHelper::getDouble(int1) / *ObjectHelper::getDouble(int2), output);
		else if(int1.getType() == "integer")
			ObjectHelper::newInteger(*ObjectHelper::getInteger(int1) / *ObjectHelper::getInteger(int2), output);
	}
	else if(text[0] == "mod")
	{
		Object int1, int2;
		evaluate(text[1], stateCopy, int1, globalMode);
		evaluate(text[2], stateCopy, int2, globalMode);

		if(int1.getType() != int2.getType())
			std::cout << "Syntax error '" << line << "'; types don't match\n";
		else if(int1.getType() == "integer" && *ObjectHelper::getInteger(int2) != 0)
		{
			int result = *ObjectHelper::getInteger(int1) % *ObjectHelper::getInteger(int2);
			if(result < 0)
				result = *ObjectHelper::getInteger(int2) + result;
			ObjectHelper::newInteger(result, output);
		}
		else
			std::cout << "Syntax error '" << line << "'; cannot mod these types\n";
	}
	else if(text[0] == "and")
	{
		Object int1, int2;
		evaluate(text[1], stateCopy, int1, globalMode);
		evaluate(text[2], stateCopy, int2, globalMode);

		if(int1.getType() != int2.getType())
			std::cout << "Syntax error '" << line << "'; types don't match\n";
		else if(int1.getType() == "integer" && int2.getType() == "integer")
		{
			int result = *ObjectHelper::getInteger(int1) & *ObjectHelper::getInteger(int2);
			ObjectHelper::newInteger(result, output);
		}
		else
			std::cout << "Syntax error '" << line << "'; cannot 'and' these types\n";
	}
	else if(text[0] == "set_element")
	{
		Object mat, row, col, value;
		evaluate(text[1], stateCopy, mat, globalMode);
		evaluate(text[2], stateCopy, row, globalMode);
		evaluate(text[3], stateCopy, col, globalMode);
		evaluate(text[4], stateCopy, value, globalMode);

		try
		{
			algebra::mat* m = new algebra::mat;
			*m = *ObjectHelper::getMatrix(mat);
			if(value.getType() == "double")
			{
				Complex c;
				c.real = *ObjectHelper::getDouble(value);
				c.imag = 0;
				m->setDirect(*ObjectHelper::getInteger(row), *ObjectHelper::getInteger(col), c);
			}
			else
				m->setDirect(*ObjectHelper::getInteger(row), *ObjectHelper::getInteger(col), *ObjectHelper::getComplex(value));
			ObjectHelper::newMatrix(m, output);
		}
		catch(std::string& e)
		{
			std::cout << "Syntax error '" << line << "': " << e << "\n";
		}
	}
	else if(text[0] == "get_element")
	{
		Object mat, row, col;
		evaluate(text[1], stateCopy, mat, globalMode);
		evaluate(text[2], stateCopy, row, globalMode);
		evaluate(text[3], stateCopy, col, globalMode);

		try
		{
			algebra::mat* m = ObjectHelper::getMatrix(mat);
			ObjectHelper::newComplex(m->getDirect(*ObjectHelper::getInteger(row),*ObjectHelper::getInteger(col)), output);
		}
		catch(std::string& e)
		{
			std::cout << "Syntax error '" << line << "': " << e << "\n";
		}
	}
	else if(text[0] == "print")
	{
		Object val;
		evaluate(text[1], stateCopy, val, globalMode);
		val.print(std::cout, true);
		std::cout << "\n";
	}
	else if(text[0] == "print_new_line")
	{
		std::cout << "\n";
	}
	else if(text[0] == "print_one_line")
	{
		Object val;
		evaluate(text[1], stateCopy, val, globalMode);
		val.print(std::cout, false);
	}
	else if(text[0] == "new_file")
	{
		Object filename;
		evaluate(text[1], stateCopy, filename, globalMode);
		std::ofstream f;
		f.open(ObjectHelper::getString(filename)->c_str());
		if(!f.is_open())
			std::cout << "Cannot open file '" << *ObjectHelper::getString(filename) << "'\n";
		else
			f.close();
	}
	else if(text[0] == "print_file")
	{
		Object filename;
		evaluate(text[1], stateCopy, filename, globalMode);
		std::ofstream f;
		f.open(ObjectHelper::getString(filename)->c_str(), std::ios_base::app);
		if(!f.is_open())
			std::cout << "Cannot open file '" << *ObjectHelper::getString(filename) << "'\n";
		else
		{
			for(int i=2; i<(int)text.size(); ++i)
			{
				Object var;
				evaluate(text[i], stateCopy, var, globalMode);
				var.print(f, false, true);
				f << ",";
			}
			f << "\n";
			f.close();
		}
	}
	else if(text[0] == "tensor")
	{
		Object int1, int2;
		evaluate(text[1], stateCopy, int1, globalMode);
		evaluate(text[2], stateCopy, int2, globalMode);

		if(int1.getType() == "matrix")
		{
			algebra::mat *A, *B;
			A = ObjectHelper::getMatrix(int1);
			B = ObjectHelper::getMatrix(int2);

			algebra::mat *C = new algebra::mat;
			Complex one = algebra::ComplexHelper::one();
			A->tensor(B, C);

			ObjectHelper::newMatrix(C, output);
		}
		else
			std::cout << "Syntax error '" << line << "'; cannot tensor these types\n";
	}
	else if(text[0] == "partial_measure")
	{
		// partial_measure psi space index ket result
		Object psi_obj, system_obj, index_obj, ket_obj, result_name;
		evaluate(text[1], stateCopy, psi_obj, globalMode);
		evaluate(text[2], stateCopy, system_obj, globalMode);
		evaluate(text[3], stateCopy, index_obj, globalMode);
		evaluate(text[4], stateCopy, ket_obj, globalMode);
		evaluate(text[5], stateCopy, result_name, globalMode);

		algebra::mat *psi = ObjectHelper::getMatrix(psi_obj);
		std::vector <int>* system = ObjectHelper::getSystem(system_obj);
		int index = *ObjectHelper::getInteger(index_obj);

		if(index < 0 || index >= (int)system->size())
		{
			std::cout << "Syntax error '" << line << "'; index out of bounds.\n";
			throw std::string("Syntax Error.");
		}

		// first find p(i) the prob. that measurement results in |i> \in H[index]
		// note: index is 0 as left most space (unlike measure_qbit)
		// p(i) = sum(v, |<i,v|psi>|^2)

		std::vector <double> p;
		std::vector <int> p_index;
		p.resize((*system)[index]);
		p_index.resize(system->size());
		algebra::mat m, B, temp;
		int systemSize=1;
		double pSq = 0;
		for(int i=0; i<(int)p_index.size(); ++i)
		{
			if(i != index)
				systemSize *= (*system)[i];
			p_index[i] = 0;
		}
		for(int i=0; i<(int)p.size(); ++i)
		{
			p_index[index] = i;
			for(int k=0; k<systemSize; ++k)
			{
				m.create(1);
				m(0,0) = algebra::ComplexHelper::one();
				for(int j=0; j<(int)p_index.size(); ++j)
				{
					B.create((*system)[j], 1);
					B.zero();
					B(p_index[j], 0) = algebra::ComplexHelper::one();
					m.tensor(&B, &temp);
					m = temp;
				}
				Complex sum;
				m.dot(psi, sum);
				algebra::ComplexHelper::sq(sum);
				p[i] += sum.real;
				for(int j=0; j<(int)p_index.size(); ++j)
				{
					if(j == index)
						continue;
					p_index[j] ++;
					if(p_index[j] >= (*system)[j])
						p_index[j] = 0;
					else
						break;
				}
			}
		//	std::cout << "p[" << i << "] = " << p[i] << "\n";
			pSq += p[i];
			p[i] *= 100.0;
		}
		pSq = sqrt(pSq);
		double last = 0;
		std::vector <double> pSave;
		for(int i=0; i<(int)p.size(); ++i)
		{
			pSave.push_back(p[i]/100.0);
			if(p[i] == 0)
				p[i] = -1;
			else
			{
				p[i] += last;
				last = p[i];
			}
			std::cout << "p[" << i << "] = " << p[i] << "\n";
		}

		int r = rand()%100;
		int result = -1;
		for(int i=0; i<(int)p.size(); ++i)
		{
			if((double)r < p[i])
			{
				result = i;
				break;
			}
		}
		if(result == -1)
		{
			for(int i=(int)p.size()-1; i>=0; --i)
			{
				if(p[i] > 0)
				{
					result = i;
					break;
				}
			}
		}
		if(result == -1)
			throw std::string("Measurement error.\n");

		std::cout << "result = " << result << "\n";

		algebra::mat *m2 = new algebra::mat;
		m2->create(psi->getRows(), 1);
		m2->zero();
		for(int i=0; i<(int)p_index.size(); ++i)
		{
			//if(i != index)
			//	systemSize *= (*system)[i];
			p_index[i] = 0;
		}
		p_index[index] = result;
		pSq = sqrt(pSave[result]);
		std::cout << "pSq = " << pSq << "\n";
		for(int k=0; k<systemSize; ++k)
		{
			m.create(1);
			m(0,0) = algebra::ComplexHelper::one();
			for(int j=0; j<(int)p_index.size(); ++j)
			{
			//	std::cout << p_index[j] << ", ";
				B.create((*system)[j], 1);
				B.zero();
				B(p_index[j], 0) = algebra::ComplexHelper::one();
				m.tensor(&B, &temp);
				m = temp;
			}
			Complex sum;
			m.dot(psi, sum);
			//std::cout << "= " << sum.real << "\n";
			sum.real /= pSq;
			sum.imag /= pSq;
			m.multiplyScalar(sum);
			m2->add(&m, &temp);
			*m2 = temp;
			for(int j=0; j<(int)p_index.size(); ++j)
			{
				if(j == index)
					continue;
				p_index[j] ++;
				if(p_index[j] >= (*system)[j])
					p_index[j] = 0;
				else
					break;
			}
		}

		ObjectHelper::newMatrix(m2, output);
	}
	else if(text[0] == "measure_distribution")
	{
		// measure_distribution psi space index result
		Object psi_obj, system_obj, index_obj, result_name;
		evaluate(text[1], stateCopy, psi_obj, globalMode);
		evaluate(text[2], stateCopy, system_obj, globalMode);
		evaluate(text[3], stateCopy, index_obj, globalMode);
		evaluate(text[4], stateCopy, result_name, globalMode);

		algebra::mat *psi = ObjectHelper::getMatrix(psi_obj);
		std::vector <int>* system = ObjectHelper::getSystem(system_obj);
		int index = *ObjectHelper::getInteger(index_obj);

		if(index < 0 || index >= (int)system->size())
		{
			std::cout << "Syntax error '" << line << "'; index out of bounds.\n";
			throw std::string("Syntax Error.");
		}

		// first find p(i) the prob. that measurement results in |i> \in H[index]
		// note: index is 0 as left most space (unlike measure_qbit)
		// p(i) = sum(v, |<i,v|psi>|^2)

		std::vector <double> p;
		std::vector <int> p_index;
		p.resize((*system)[index]);
		p_index.resize(system->size());
		algebra::mat m, B, temp;
		int systemSize=1;
		double pSq = 0;
		for(int i=0; i<(int)p_index.size(); ++i)
		{
			if(i != index)
				systemSize *= (*system)[i];
			p_index[i] = 0;
		}
		Object temp_obj;
		for(int i=0; i<(int)p.size(); ++i)
		{
			p_index[index] = i;
			for(int k=0; k<systemSize; ++k)
			{
				m.create(1);
				m(0,0) = algebra::ComplexHelper::one();
				for(int j=0; j<(int)p_index.size(); ++j)
				{
					B.create((*system)[j], 1);
					B.zero();
					B(p_index[j], 0) = algebra::ComplexHelper::one();
					m.tensor(&B, &temp);
					m = temp;
				}
				Complex sum;
				m.dot(psi, sum);
				algebra::ComplexHelper::sq(sum);
				p[i] += sum.real;
				for(int j=0; j<(int)p_index.size(); ++j)
				{
					if(j == index)
						continue;
					p_index[j] ++;
					if(p_index[j] >= (*system)[j])
						p_index[j] = 0;
					else
						break;
				}
			}
			std::stringstream str;
			str << *(ObjectHelper::getString(result_name)) << "." << i;
			ObjectHelper::newDouble(p[i], temp_obj);
			ourState->set(str.str(), temp_obj);
		}

		ObjectHelper::newVector(p, output);
	}
	else if(text[0] == "measure")
	{
		// measure psi space index result
		Object psi_obj, system_obj, index_obj, result_name;
		evaluate(text[1], stateCopy, psi_obj, globalMode);
		evaluate(text[2], stateCopy, system_obj, globalMode);
		evaluate(text[3], stateCopy, index_obj, globalMode);
		evaluate(text[4], stateCopy, result_name, globalMode);

		algebra::mat *psi = ObjectHelper::getMatrix(psi_obj);
		std::vector <int>* system = ObjectHelper::getSystem(system_obj);
		int index = *ObjectHelper::getInteger(index_obj);

		if(index < 0 || index >= (int)system->size())
		{
			std::cout << "Syntax error '" << line << "'; index out of bounds.\n";
			throw std::string("Syntax Error.");
		}

		// first find p(i) the prob. that measurement results in |i> \in H[index]
		// note: index is 0 as left most space (unlike measure_qbit)
		// p(i) = sum(v, |<i,v|psi>|^2)

		std::vector <double> p;
		std::vector <int> p_index;
		p.resize((*system)[index]);	// set to 0?
		p_index.resize(system->size());
		algebra::mat m, B, temp;
		int systemSize=1;
		double pSq = 0;
		for(int i=0; i<(int)p_index.size(); ++i)
		{
			if(i != index)
				systemSize *= (*system)[i];
			p_index[i] = 0;
		}
		for(int i=0; i<(int)p.size(); ++i)
		{
			p_index[index] = i;
			for(int k=0; k<systemSize; ++k)
			{
				m.create(1);
				m(0,0) = algebra::ComplexHelper::one();
				for(int j=0; j<(int)p_index.size(); ++j)
				{
					B.create((*system)[j], 1);
					B.zero();
					B(p_index[j], 0) = algebra::ComplexHelper::one();
					m.tensor(&B, &temp);
					m = temp;
				}
				Complex sum;
				m.dot(psi, sum);
				algebra::ComplexHelper::sq(sum);
				p[i] += sum.real;
				for(int j=0; j<(int)p_index.size(); ++j)
				{
					if(j == index)
						continue;
					p_index[j] ++;
					if(p_index[j] >= (*system)[j])
						p_index[j] = 0;
					else
						break;
				}
			}
		//	std::cout << "p[" << i << "] = " << p[i] << "\n";
			pSq += p[i];
			p[i] *= 100.0;
		}
		pSq = sqrt(pSq);
		double last = 0;
		std::vector <double> pSave;
		for(int i=0; i<(int)p.size(); ++i)
		{
			pSave.push_back(p[i]/100.0);
			if(p[i] == 0)
				p[i] = -1;
			else
			{
				p[i] += last;
				last = p[i];
			}
			//std::cout << "p[" << i << "] = " << p[i] << "\n";
		}

		int r = rand()%10000;
		double r2 = (double)r;
		r2 = r2 / 10000.0;
		r2 *= 100.0;
		int result = -1;
		for(int i=0; i<(int)p.size(); ++i)
		{
			if(r2 < p[i])
			{
				result = i;
				break;
			}
		}
		if(result == -1)
		{
			for(int i=(int)p.size()-1; i>=0; --i)
			{
				if(p[i] > 0)
				{
					result = i;
					break;
				}
			}
		}
		if(result == -1)
			throw std::string("Measurement error.\n");

		//std::cout << "result = " << result << "\n";

		Object result_obj;
		ObjectHelper::newInteger(result, result_obj);
		ourState->set(*ObjectHelper::getString(result_name), result_obj);

		algebra::mat *m2 = new algebra::mat;
		m2->create(psi->getRows(), 1);
		m2->zero();
		for(int i=0; i<(int)p_index.size(); ++i)
		{
			//if(i != index)
			//	systemSize *= (*system)[i];
			p_index[i] = 0;
		}
		p_index[index] = result;
		pSq = sqrt(pSave[result]);
		//std::cout << "pSq = " << pSq << "\n";
		for(int k=0; k<systemSize; ++k)
		{
			m.create(1);
			m(0,0) = algebra::ComplexHelper::one();
			for(int j=0; j<(int)p_index.size(); ++j)
			{
			//	std::cout << p_index[j] << ", ";
				B.create((*system)[j], 1);
				B.zero();
				B(p_index[j], 0) = algebra::ComplexHelper::one();
				m.tensor(&B, &temp);
				m = temp;
			}
			Complex sum;
			m.dot(psi, sum);
			//std::cout << "= " << sum.real << "\n";
			sum.real /= pSq;
			sum.imag /= pSq;
			m.multiplyScalar(sum);
			m2->add(&m, &temp);
			*m2 = temp;
			for(int j=0; j<(int)p_index.size(); ++j)
			{
				if(j == index)
					continue;
				p_index[j] ++;
				if(p_index[j] >= (*system)[j])
					p_index[j] = 0;
				else
					break;
			}
		}

		ObjectHelper::newMatrix(m2, output);
	}
	else if(text[0] == "measure_qbit")
	{
		// measure psi qbit_number; should return new psi' in state measured
		Object psi_obj;
		evaluate(text[1], stateCopy, psi_obj, globalMode);
		if(psi_obj.getType() != "matrix")
		{
			std::cout << "Syntax error '" << line << "'; type not matrix.\n";
			return;
		}
		algebra::mat *psi = ObjectHelper::getMatrix(psi_obj);
		Object qubit_obj;
		evaluate(text[2], stateCopy, qubit_obj, globalMode);
		if(qubit_obj.getType() != "integer")
		{
			std::cout << "Syntax error '" << line << "'; type not integer.\n";
			return;
		}

		Object result_name;
		evaluate(text[3], stateCopy, result_name, globalMode);

		int qubit = *ObjectHelper::getInteger(qubit_obj);
		int spaceSize = psi->getRows();
		int k = 1;
		int q = 1;
		for(int i=0; i<qubit; ++i)
			q = q << 1;

		double sum = 0;

		for(int i=0; i<spaceSize; ++i)
		{
			if( (i & q) / q == 0 )
			{
				Complex c = (*psi)(i,0);
				c.imag *= -1;
				c = algebra::ComplexHelper::mult((*psi)(i,0), c);
				sum += c.real;
			}
		}

		
		int r = rand()%10000;
		double p = (double)r;
		p = p / 10000.0;
		algebra::mat *m = new algebra::mat;
		(*m) = (*psi);
		if(p <= sum)
		{
			//std::cout << "measure 0\n";
			Object measurement;
			ObjectHelper::newInteger(0, measurement);
			ourState->set(*ObjectHelper::getString(result_name), measurement);
			Complex sum2;
			sum2.real = 1.0/sqrt(sum);
			sum2.imag = 0;
			for(int i=0; i<spaceSize; ++i)
			{
				if( (i & q) / q == 0 )
					(*m)(i,0) = algebra::ComplexHelper::mult((*m)(i,0), sum2);
				else
					(*m)(i,0) = algebra::ComplexHelper::zero();
			}
		}
		else
		{
			//std::cout << "measure 1\n";
			Object measurement;
			ObjectHelper::newInteger(1, measurement);
			ourState->set(*ObjectHelper::getString(result_name), measurement);
			Complex sum2;
			sum2.real = 1.0/sqrt(1-sum);
			sum2.imag = 0;
			for(int i=0; i<spaceSize; ++i)
			{
				if( (i & q) / q == 0 )
					(*m)(i,0) = algebra::ComplexHelper::zero();
				else
					(*m)(i,0) = algebra::ComplexHelper::mult((*psi)(i,0), sum2);
			}
		}

		ObjectHelper::newMatrix(m, output);
	}
	else if(text[0] == "help")
	{
		Object command_name_obj;
		evaluate(text[1], stateCopy, command_name_obj, globalMode);
		std::string* command_name = ObjectHelper::getString(command_name_obj);
		bool found = false;
		for(unsigned int i=0; i<registeredCommands.size(); ++i)
		{
			if(!registeredCommands[i]) continue;
			if(registeredCommands[i]->getName() == *command_name)
			{
				registeredCommands[i]->printHelp();
				break;
			}
		}
	}
	else
	{
		// search through registered commands:
		bool found = false;
		for(unsigned int i=0; i<registeredCommands.size(); ++i)
		{
			if(!registeredCommands[i]) continue;
			if(registeredCommands[i]->getName() == text[0])
			{
				registeredCommands[i]->execute(text, stateCopy, output, globalMode, copyState, this);
				found = true;
				break;
			}
		}
		if(!found)
			std::cout << "Syntax error '" << line << "'\n";
	}
}
}