/*
OpenGL.h
Written by Walter O. Krawec

Copyright (c) 2013 Walter O. Krawec

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef WOK_OPENGL_H_
#define WOK_OPENGL_H_

#ifdef WIN32
#include <windows.h>		// Header File For Windows
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <string>

#pragma comment(lib, "OpenGL32.lib")
#pragma comment(lib, "Glu32.lib")
#pragma comment(lib, "gdiplus.lib")
#else
#include <gl.h>
#include <glu.h>
#include <glut.h>
#endif

namespace wok
{
	namespace gl
	{
		class Camera
		{
		public:
			Camera() : x(0), y(0), z(0) {};
			Camera(float _x, float _y, float _z) : x(_x), y(_y), z(_z) {};

			void select()
			{
				glTranslatef(x,y,z);
			}

			float& xPos() {return x;}
			float& yPos() {return y;}
			float& zPos() {return z;}
		private:
			float x,y,z;
		};

		class glWindow
		{
		public:
#ifdef WIN32
			glWindow() : hDC(NULL), hRC(NULL), hWnd(NULL), fullscreen(false) {};
#else
		        glWindow() : fullscreen(false){};
#endif
			bool initGL()
			{
				glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
				glClearColor(0.0f, 0.0f, 0.0f, 0.5f);				// Black Background
				glClearDepth(1.0f);									// Depth Buffer Setup
				glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
				glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do
				glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations

				return true;
			};

			bool resizeScreen(int width, int height)
			{
				if(height == 0 || width == 0)
					return false;

				glViewport(0,0,width,height);						// Reset The Current Viewport

				glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
				glLoadIdentity();									// Reset The Projection Matrix

				// Calculate The Aspect Ratio Of The Window
				gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);

				glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
				glLoadIdentity();	

				return true;
			}

			void beginDraw()
			{
				glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear Screen And Depth Buffer
				glLoadIdentity();									// Reset The Current Modelview Matrix
			}
			void endDraw()
			{
#ifdef WIN32
				SwapBuffers(hDC);
#else
				glutSwapBuffers();
#endif
			}

#ifdef WIN32
			bool createWindow(WNDPROC WndProc, const std::string& title, int width, int height, int bits, bool fullscreenflag);
			void destroyWindow();
			void wndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
#endif

			bool getKey(unsigned int keyIndex, bool resetKey=false)
			{
				if(keyIndex >= 256)
					return false;
				bool value = keys[keyIndex];
				if(resetKey)
					keys[keyIndex] = false;
				return value;
			}
			void setKey(unsigned char keyIndex, bool value)
			{
				keys[keyIndex] = value;
			}
		private:
#ifdef WIN32
			HDC			hDC;		// Private GDI Device Context
			HGLRC		hRC;		// Permanent Rendering Context
			HWND		hWnd;		// Holds Our Window Handle
			HINSTANCE	hInstance;		// Holds The Instance Of The Application
#endif

			bool fullscreen;
			bool keys[256];
		};
	};
}

#endif
